@@ -233,20 +233,44 @@ enum {
 	TGSI_SEMANTIC_DEFAULT_TESSINNER_SI,
 };
 
+/* For VS shader key fix_fetch. */
+enum {
+	SI_FIX_FETCH_NONE = 0,
+	SI_FIX_FETCH_A2_SNORM,
+	SI_FIX_FETCH_A2_SSCALED,
+	SI_FIX_FETCH_A2_SINT,
+	SI_FIX_FETCH_RGBA_32_UNORM,
+	SI_FIX_FETCH_RGBX_32_UNORM,
+	SI_FIX_FETCH_RGBA_32_SNORM,
+	SI_FIX_FETCH_RGBX_32_SNORM,
+	SI_FIX_FETCH_RGBA_32_USCALED,
+	SI_FIX_FETCH_RGBA_32_SSCALED,
+	SI_FIX_FETCH_RGBA_32_FIXED,
+	SI_FIX_FETCH_RGBX_32_FIXED,
+};
+
 struct si_shader;
 
+/* State of the context creating the shader object. */
+struct si_compiler_ctx_state {
+	/* Should only be used by si_init_shader_selector_async and
+	 * si_build_shader_variant if thread_index == -1 (non-threaded). */
+	LLVMTargetMachineRef		tm;
+
+	/* Used if thread_index == -1 or if debug.async is true. */
+	struct pipe_debug_callback	debug;
+
+	/* Used for creating the log string for gallium/ddebug. */
+	bool				is_debug_context;
+};
+
 /* A shader selector is a gallium CSO and contains shader variants and
  * binaries for one TGSI program. This can be shared by multiple contexts.
  */
 struct si_shader_selector {
 	struct si_screen	*screen;
 	struct util_queue_fence ready;
-
-	/* Should only be used by si_init_shader_selector_async
-	 * if thread_index == -1 (non-threaded). */
-	LLVMTargetMachineRef	tm;
-	struct pipe_debug_callback debug;
-	bool			is_debug_context;
+	struct si_compiler_ctx_state compiler_ctx_state;
 
 	pipe_mutex		mutex;
 	struct si_shader	*first_variant; /* immutable after the first variant */
@@ -257,6 +281,8 @@ struct si_shader_selector {
 	 */
 	struct si_shader	*main_shader_part;
 
+	struct si_shader	*gs_copy_shader;
+
 	struct tgsi_token       *tokens;
 	struct pipe_stream_output_info  so;
 	struct tgsi_shader_info		info;
@@ -285,9 +311,12 @@ struct si_shader_selector {
 	/* CS parameters */
 	unsigned local_size;
 
-	/* masks of "get_unique_index" bits */
-	uint64_t	outputs_written;
-	uint32_t	patch_outputs_written;
+	uint64_t	outputs_written;	/* "get_unique_index" bits */
+	uint32_t	patch_outputs_written;	/* "get_unique_index" bits */
+	uint32_t	outputs_written2;	/* "get_unique_index2" bits */
+
+	uint64_t	inputs_read;		/* "get_unique_index" bits */
+	uint32_t	inputs_read2;		/* "get_unique_index2" bits */
 };
 
 /* Valid shader configurations:
@@ -309,18 +338,15 @@ struct si_vs_prolog_bits {
 /* Common VS bits between the shader key and the epilog key. */
 struct si_vs_epilog_bits {
 	unsigned	export_prim_id:1; /* when PS needs it and GS is disabled */
-	/* TODO:
-	 * - skip clipdist, culldist (including clipvertex code) exports based
-	 *   on which clip_plane_enable bits are set
-	 * - skip layer, viewport, clipdist, and culldist parameter exports
-	 *   if PS doesn't read them
-	 */
 };
 
 /* Common TCS bits between the shader key and the epilog key. */
 struct si_tcs_epilog_bits {
 	unsigned	prim_mode:3;
-	uint64_t	inputs_to_copy;
+};
+
+struct si_gs_prolog_bits {
+	unsigned	tri_strip_adj_fix:1;
 };
 
 /* Common PS bits between the shader key and the prolog key. */
@@ -340,6 +366,7 @@ struct si_ps_prolog_bits {
 struct si_ps_epilog_bits {
 	unsigned	spi_shader_col_format;
 	unsigned	color_is_int8:8;
+	unsigned	color_is_int10:8;
 	unsigned	last_cbuf:3;
 	unsigned	alpha_func:3;
 	unsigned	alpha_to_one:1;
@@ -360,6 +387,9 @@ union si_shader_part_key {
 	struct {
 		struct si_tcs_epilog_bits states;
 	} tcs_epilog;
+	struct {
+		struct si_gs_prolog_bits states;
+	} gs_prolog;
 	struct {
 		struct si_ps_prolog_bits states;
 		unsigned	num_input_sgprs:5;
@@ -381,24 +411,53 @@ union si_shader_part_key {
 	} ps_epilog;
 };
 
-union si_shader_key {
-	struct {
-		struct si_ps_prolog_bits prolog;
-		struct si_ps_epilog_bits epilog;
-	} ps;
-	struct {
-		struct si_vs_prolog_bits prolog;
-		struct si_vs_epilog_bits epilog;
-		unsigned	as_es:1; /* export shader */
-		unsigned	as_ls:1; /* local shader */
-	} vs;
-	struct {
-		struct si_tcs_epilog_bits epilog;
-	} tcs; /* tessellation control shader */
-	struct {
-		struct si_vs_epilog_bits epilog; /* same as VS */
-		unsigned	as_es:1; /* export shader */
-	} tes; /* tessellation evaluation shader */
+struct si_shader_key {
+	/* Prolog and epilog flags. */
+	union {
+		struct {
+			struct si_ps_prolog_bits prolog;
+			struct si_ps_epilog_bits epilog;
+		} ps;
+		struct {
+			struct si_vs_prolog_bits prolog;
+			struct si_vs_epilog_bits epilog;
+		} vs;
+		struct {
+			struct si_tcs_epilog_bits epilog;
+		} tcs; /* tessellation control shader */
+		struct {
+			struct si_vs_epilog_bits epilog; /* same as VS */
+		} tes; /* tessellation evaluation shader */
+		struct {
+			struct si_gs_prolog_bits prolog;
+		} gs;
+	} part;
+
+	/* These two are initially set according to the NEXT_SHADER property,
+	 * or guessed if the property doesn't seem correct.
+	 */
+	unsigned as_es:1; /* export shader */
+	unsigned as_ls:1; /* local shader */
+
+	/* Flags for monolithic compilation only. */
+	union {
+		struct {
+			/* One nibble for every input: SI_FIX_FETCH_* enums. */
+			uint64_t	fix_fetch;
+		} vs;
+		struct {
+			uint64_t	inputs_to_copy; /* for fixed-func TCS */
+		} tcs;
+	} mono;
+
+	/* Optimization flags for asynchronous compilation only. */
+	union {
+		struct {
+			uint64_t	kill_outputs; /* "get_unique_index" bits */
+			uint32_t	kill_outputs2; /* "get_unique_index2" bits */
+			unsigned	clip_disable:1;
+		} hw_vs; /* HW VS (it can be VS, TES, GS) */
+	} opt;
 };
 
 struct si_shader_config {
@@ -406,6 +465,7 @@ struct si_shader_config {
 	unsigned			num_vgprs;
 	unsigned			spilled_sgprs;
 	unsigned			spilled_vgprs;
+	unsigned			private_mem_vgprs;
 	unsigned			lds_size;
 	unsigned			spi_ps_input_ena;
 	unsigned			spi_ps_input_addr;
@@ -415,6 +475,18 @@ struct si_shader_config {
 	unsigned			rsrc2;
 };
 
+enum {
+	/* SPI_PS_INPUT_CNTL_i.OFFSET[0:4] */
+	EXP_PARAM_OFFSET_0 = 0,
+	EXP_PARAM_OFFSET_31 = 31,
+	/* SPI_PS_INPUT_CNTL_i.DEFAULT_VAL[0:1] */
+	EXP_PARAM_DEFAULT_VAL_0000 = 64,
+	EXP_PARAM_DEFAULT_VAL_0001,
+	EXP_PARAM_DEFAULT_VAL_1110,
+	EXP_PARAM_DEFAULT_VAL_1111,
+	EXP_PARAM_UNDEFINED = 255,
+};
+
 /* GCN-specific shader info. */
 struct si_shader_info {
 	ubyte			vs_output_param_offset[SI_MAX_VS_OUTPUTS];
@@ -427,18 +499,24 @@ struct si_shader_info {
 };
 
 struct si_shader {
+	struct si_compiler_ctx_state	compiler_ctx_state;
+
 	struct si_shader_selector	*selector;
 	struct si_shader		*next_variant;
 
 	struct si_shader_part		*prolog;
 	struct si_shader_part		*epilog;
 
-	struct si_shader		*gs_copy_shader;
 	struct si_pm4_state		*pm4;
 	struct r600_resource		*bo;
 	struct r600_resource		*scratch_bo;
-	union si_shader_key		key;
+	struct si_shader_key		key;
+	struct util_queue_fence		optimized_ready;
+	bool				compilation_failed;
+	bool				is_monolithic;
+	bool				is_optimized;
 	bool				is_binary_shared;
+	bool				is_gs_copy_shader;
 
 	/* The following data is all that's needed for binary shaders. */
 	struct radeon_shader_binary	binary;
@@ -460,6 +538,11 @@ struct si_shader_part {
 };
 
 /* si_shader.c */
+struct si_shader *
+si_generate_gs_copy_shader(struct si_screen *sscreen,
+			   LLVMTargetMachineRef tm,
+			   struct si_shader_selector *gs_selector,
+			   struct pipe_debug_callback *debug);
 int si_compile_tgsi_shader(struct si_screen *sscreen,
 			   LLVMTargetMachineRef tm,
 			   struct si_shader *shader,
@@ -478,10 +561,11 @@ int si_compile_llvm(struct si_screen *sscreen,
 		    const char *name);
 void si_shader_destroy(struct si_shader *shader);
 unsigned si_shader_io_get_unique_index(unsigned semantic_name, unsigned index);
+unsigned si_shader_io_get_unique_index2(unsigned name, unsigned index);
 int si_shader_binary_upload(struct si_screen *sscreen, struct si_shader *shader);
 void si_shader_dump(struct si_screen *sscreen, struct si_shader *shader,
 		    struct pipe_debug_callback *debug, unsigned processor,
-		    FILE *f);
+		    FILE *f, bool check_debug_option);
 void si_multiwave_lds_size_workaround(struct si_screen *sscreen,
 				      unsigned *lds_size);
 void si_shader_apply_scratch_relocs(struct si_context *sctx,
